@@ -8,10 +8,6 @@ TODO
 
 """
 
-from __future__ import print_function
-from __future__ import division
-from builtins import str, range
-
 import os
 import glob
 
@@ -26,11 +22,13 @@ from numpy import mean, zeros, arange
 from numpy.random import normal
 
 from scipy.interpolate import interp1d
+
 try:
     from scipy.interpolate import RegularGridInterpolator
 except ImportError:
     pass
 
+
 def resize3DField(var, data, coordsAndSizesTuple, method, mute):
     """Resize 3D fields
 
@@ -45,19 +43,31 @@ def resize3DField(var, data, coordsAndSizesTuple, method, mute):
     """
 
     # Unpack the tuple for better readability
-    xCoordOld, yCoordOld, zCoordOld,\
-        xCoordNew, yCoordNew, zCoordNew,\
-        newNx, newNy, newNz = coordsAndSizesTuple
+    (
+        xCoordOld,
+        yCoordOld,
+        zCoordOld,
+        xCoordNew,
+        yCoordNew,
+        zCoordNew,
+        newNx,
+        newNy,
+        newNz,
+    ) = coordsAndSizesTuple
 
-    if not(mute):
-        print("    Resizing "+var +
-              ' to (nx,ny,nz) = ({},{},{})'.format(newNx, newNy, newNz))
+    if not (mute):
+        print(
+            "    Resizing "
+            + var
+            + " to (nx,ny,nz) = ({},{},{})".format(newNx, newNy, newNz)
+        )
 
     # Make the regular grid function (see examples in
     # https://docs.scipy.org/doc/scipy/reference/generated/scipy.interpolate.RegularGridInterpolator.html
     # for details)
     gridInterpolator = RegularGridInterpolator(
-        (xCoordOld, yCoordOld, zCoordOld), data, method)
+        (xCoordOld, yCoordOld, zCoordOld), data, method
+    )
 
     # Need to fill with one exrta z plane (will only contain zeros)
     newData = np.zeros((newNx, newNy, newNz))
@@ -71,9 +81,20 @@ def resize3DField(var, data, coordsAndSizesTuple, method, mute):
     return var, newData
 
 
-def resize(newNx, newNy, newNz, mxg=2, myg=2,
-           path="data", output="./", informat="nc", outformat=None,
-           method='linear', maxProc=None, mute=False):
+def resize(
+    newNx,
+    newNy,
+    newNz,
+    mxg=2,
+    myg=2,
+    path="data",
+    output="./",
+    informat="nc",
+    outformat=None,
+    method="linear",
+    maxProc=None,
+    mute=False,
+):
     """Increase/decrease the number of points in restart files.
 
     NOTE: Can't overwrite
@@ -115,7 +136,7 @@ def resize(newNx, newNy, newNz, mxg=2, myg=2,
 
     if method is None:
         # Make sure the method is set
-        method = 'linear'
+        method = "linear"
 
     if outformat is None:
         outformat = informat
@@ -126,13 +147,13 @@ def resize(newNx, newNy, newNz, mxg=2, myg=2,
 
     def is_pow2(x):
         """Returns true if x is a power of 2"""
-        return (x > 0) and ((x & (x-1)) == 0)
+        return (x > 0) and ((x & (x - 1)) == 0)
 
     if not is_pow2(newNz):
         print("ERROR: New Z size {} must be a power of 2".format(newNz))
         return False
 
-    file_list = glob.glob(os.path.join(path, "BOUT.restart.*."+informat))
+    file_list = glob.glob(os.path.join(path, "BOUT.restart.*." + informat))
     file_list.sort()
     nfiles = len(file_list)
 
@@ -140,12 +161,12 @@ def resize(newNx, newNy, newNz, mxg=2, myg=2,
         print("ERROR: No data found in {}".format(path))
         return False
 
-    if not(mute):
+    if not (mute):
         print("Number of files found: " + str(nfiles))
 
     for f in file_list:
-        new_f = os.path.join(output, f.split('/')[-1])
-        if not(mute):
+        new_f = os.path.join(output, f.split("/")[-1])
+        if not (mute):
             print("Changing {} => {}".format(f, new_f))
 
         # Open the restart file in read mode and create the new file
@@ -179,9 +200,17 @@ def resize(newNx, newNy, newNz, mxg=2, myg=2,
             # List of jobs and results
             jobs = []
             # Pack input to resize3DField together
-            coordsAndSizesTuple = (xCoordOld, yCoordOld, zCoordOld,
-                                   xCoordNew, yCoordNew, zCoordNew,
-                                   newNx, newNy, newNz)
+            coordsAndSizesTuple = (
+                xCoordOld,
+                yCoordOld,
+                zCoordOld,
+                xCoordNew,
+                yCoordNew,
+                zCoordNew,
+                newNx,
+                newNy,
+                newNz,
+            )
 
             # Loop over the variables in the old file
             for var in old.list():
@@ -193,22 +222,32 @@ def resize(newNx, newNy, newNz, mxg=2, myg=2,
                 if old.ndims(var) == 3:
 
                     # Asynchronous call (locks first at .get())
-                    jobs.append(pool.apply_async(resize3DField,
-                                                 args=(var, data, coordsAndSizesTuple, method, mute, )))
+                    jobs.append(
+                        pool.apply_async(
+                            resize3DField,
+                            args=(
+                                var,
+                                data,
+                                coordsAndSizesTuple,
+                                method,
+                                mute,
+                            ),
+                        )
+                    )
 
                 else:
-                    if not(mute):
-                        print("    Copying "+var)
+                    if not (mute):
+                        print("    Copying " + var)
                         newData = data.copy()
-                    if not(mute):
-                        print("Writing "+var)
+                    if not (mute):
+                        print("Writing " + var)
                     new.write(var, newData)
 
             for job in jobs:
                 var, newData = job.get()
                 newData = BoutArray(newData, attributes=attributes)
-                if not(mute):
-                    print("Writing "+var)
+                if not (mute):
+                    print("Writing " + var)
                 new.write(var, newData)
 
             # Close the pool of workers
@@ -260,13 +299,13 @@ def resizeZ(newNz, path="data", output="./", informat="nc", outformat=None):
 
     def is_pow2(x):
         """Returns true if x is a power of 2"""
-        return (x > 0) and ((x & (x-1)) == 0)
+        return (x > 0) and ((x & (x - 1)) == 0)
 
     if not is_pow2(newNz):
         print("ERROR: New Z size must be a power of 2")
         return False
 
-    file_list = glob.glob(os.path.join(path, "BOUT.restart.*."+informat))
+    file_list = glob.glob(os.path.join(path, "BOUT.restart.*." + informat))
     file_list.sort()
     nfiles = len(file_list)
 
@@ -277,12 +316,11 @@ def resizeZ(newNz, path="data", output="./", informat="nc", outformat=None):
     print("Number of files found: " + str(nfiles))
 
     for f in file_list:
-        new_f = os.path.join(output, f.split('/')[-1])
+        new_f = os.path.join(output, f.split("/")[-1])
         print("Changing {} => {}".format(f, new_f))
 
         # Open the restart file in read mode and create the new file
-        with DataFile(f) as old,\
-                DataFile(new_f, write=True, create=True) as new:
+        with DataFile(f) as old, DataFile(new_f, write=True, create=True) as new:
             # Loop over the variables in the old file
             for var in old.list():
                 # Read the data
@@ -291,7 +329,7 @@ def resizeZ(newNz, path="data", output="./", informat="nc", outformat=None):
 
                 # Find 3D variables
                 if old.ndims(var) == 3:
-                    print("    Resizing "+var)
+                    print("    Resizing " + var)
 
                     nx, ny, nz = data.shape
 
@@ -305,11 +343,11 @@ def resizeZ(newNz, path="data", output="./", informat="nc", outformat=None):
 
                             # Copy coefficients across (ignoring Nyquist)
                             f_new[0] = f_old[0]  # DC
-                            for m in range(1, int(nz/2)):
+                            for m in range(1, int(nz / 2)):
                                 # + ve frequencies
                                 f_new[m] = f_old[m]
                                 # - ve frequencies
-                                f_new[newNz-m] = f_old[nz-m]
+                                f_new[newNz - m] = f_old[nz - m]
 
                             # Invert fft
                             newdata[x, y, :] = np.fft.ifft(f_new).real
@@ -322,9 +360,9 @@ def resizeZ(newNz, path="data", output="./", informat="nc", outformat=None):
                     # 1/N is the inverse transform in np.fft
                     # Note that ifft(fft(a)) = a for the same number of
                     # points in both IDL and np.ftt
-                    newdata *= (newNz/nz)
+                    newdata *= newNz / nz
                 else:
-                    print("    Copying "+var)
+                    print("    Copying " + var)
                     newdata = data.copy()
 
                 newdata = BoutArray(newdata, attributes=attributes)
@@ -362,13 +400,13 @@ def addnoise(path=".", var=None, scale=1e-5):
             if var is None:
                 for v in d.list():
                     if d.ndims(v) == 3:
-                        print(" -> "+v)
+                        print(" -> " + v)
                         data = d.read(v, asBoutArray=True)
                         data += normal(scale=scale, size=data.shape)
                         d.write(v, data)
             else:
                 # Modify a single variable
-                print(" -> "+var)
+                print(" -> " + var)
                 data = d.read(var)
                 data += normal(scale=scale, size=data.shape)
                 d.write(var, data)
@@ -402,7 +440,9 @@ def scalevar(var, factor, path="."):
             d[var] = d[var] * factor
 
 
-def create(averagelast=1, final=-1, path="data", output="./", informat="nc", outformat=None):
+def create(
+    averagelast=1, final=-1, path="data", output="./", informat="nc", outformat=None
+):
     """Create restart files from data (dmp) files.
 
     Parameters
@@ -426,15 +466,15 @@ def create(averagelast=1, final=-1, path="data", output="./", informat="nc", out
     if outformat is None:
         outformat = informat
 
-    file_list = glob.glob(os.path.join(path, "BOUT.dmp.*."+informat))
+    file_list = glob.glob(os.path.join(path, "BOUT.dmp.*." + informat))
     nfiles = len(file_list)
 
     print(("Number of data files: ", nfiles))
 
     for i in range(nfiles):
         # Open each data file
-        infname = os.path.join(path, "BOUT.dmp."+str(i)+"."+informat)
-        outfname = os.path.join(output, "BOUT.restart."+str(i)+"."+outformat)
+        infname = os.path.join(path, "BOUT.dmp." + str(i) + "." + informat)
+        outfname = os.path.join(output, "BOUT.restart." + str(i) + "." + outformat)
 
         print((infname, " -> ", outfname))
 
@@ -475,8 +515,7 @@ def create(averagelast=1, final=-1, path="data", output="./", informat="nc", out
                 if averagelast == 1:
                     slice = data[final, :, :, :]
                 else:
-                    slice = mean(data[(final - averagelast)
-                                 :final, :, :, :], axis=0)
+                    slice = mean(data[(final - averagelast) : final, :, :, :], axis=0)
 
                 print(slice.shape)
 
@@ -486,7 +525,16 @@ def create(averagelast=1, final=-1, path="data", output="./", informat="nc", out
         outfile.close()
 
 
-def redistribute(npes, path="data", nxpe=None, output=".", informat=None, outformat=None, mxg=2, myg=2):
+def redistribute(
+    npes,
+    path="data",
+    nxpe=None,
+    output=".",
+    informat=None,
+    outformat=None,
+    mxg=None,
+    myg=None,
+):
     """Resize restart files across NPES processors.
 
     Does not check if new processor arrangement is compatible with the
@@ -515,6 +563,9 @@ def redistribute(npes, path="data", nxpe=None, output=".", informat=None, outfor
         Specify file format of new restart files (must be a suffix
         understood by DataFile, e.g. 'nc'). Default is to use the same
         as informat.
+    mxg, myg : int, optional
+        Number of guard cells in x, y to use in the output (default:
+        keep the same as input)
 
     Returns
     -------
@@ -537,7 +588,7 @@ def redistribute(npes, path="data", nxpe=None, output=".", informat=None, outfor
     if informat is None:
         file_list = glob.glob(os.path.join(path, "BOUT.restart.*"))
     else:
-        file_list = glob.glob(os.path.join(path, "BOUT.restart.*."+informat))
+        file_list = glob.glob(os.path.join(path, "BOUT.restart.*." + informat))
 
     nfiles = len(file_list)
 
@@ -551,8 +602,12 @@ def redistribute(npes, path="data", nxpe=None, output=".", informat=None, outfor
         return False
 
     old_processor_layout = get_processor_layout(f, has_t_dimension=False)
-    print("Grid sizes: ", old_processor_layout.nx,
-          old_processor_layout.ny, old_processor_layout.mz)
+    print(
+        "Grid sizes: ",
+        old_processor_layout.nx,
+        old_processor_layout.ny,
+        old_processor_layout.mz,
+    )
 
     if nfiles != old_processor_layout.npes:
         print("WARNING: Number of restart files inconsistent with NPES")
@@ -569,20 +624,18 @@ def redistribute(npes, path="data", nxpe=None, output=".", informat=None, outfor
 
     try:
         new_processor_layout = create_processor_layout(
-            old_processor_layout, npes, nxpe=nxpe)
+            old_processor_layout, npes, nxpe=nxpe
+        )
     except ValueError as e:
         print("Could not find valid processor split. " + e.what())
 
-    nx = old_processor_layout.nx
-    ny = old_processor_layout.ny
-    mz = old_processor_layout.mz
-    mxg = old_processor_layout.mxg
-    myg = old_processor_layout.myg
-    old_npes = old_processor_layout.npes
-    old_nxpe = old_processor_layout.nxpe
-    old_nype = old_processor_layout.nype
-    old_mxsub = old_processor_layout.mxsub
-    old_mysub = old_processor_layout.mysub
+    old_mxg = old_processor_layout.mxg
+    old_myg = old_processor_layout.myg
+
+    if mxg is None:
+        mxg = old_mxg
+    if myg is None:
+        myg = old_myg
 
     nxpe = new_processor_layout.nxpe
     nype = new_processor_layout.nype
@@ -592,23 +645,68 @@ def redistribute(npes, path="data", nxpe=None, output=".", informat=None, outfor
 
     outfile_list = []
     for i in range(npes):
-        outpath = os.path.join(output, "BOUT.restart."+str(i)+"."+outformat)
+        outpath = os.path.join(output, "BOUT.restart." + str(i) + "." + outformat)
         outfile_list.append(DataFile(outpath, write=True, create=True))
 
     DataFileCache = create_cache(path, "BOUT.restart")
 
     for v in var_list:
         dimensions = f.dimensions(v)
-        ndims = len(dimensions)
 
         # collect data
-        data = collect(v, xguards=True, yguards=True, info=False,
-                datafile_cache=DataFileCache)
+        data = collect(
+            v, xguards=True, yguards=True, info=False, datafile_cache=DataFileCache
+        )
 
         # write data
         for i in range(npes):
             ix = i % nxpe
-            iy = int(i/nxpe)
+            iy = int(i / nxpe)
+
+            def get_block(data):
+                if mxg > old_mxg or myg > old_myg:
+                    # need to make a new array as some boundary cell points are not
+                    # present in data
+                    new_shape = list(data.shape)
+                    new_shape[0] = mxsub + 2 * mxg
+                    new_shape[1] = mysub + 2 * myg
+                    result = np.zeros(new_shape)
+
+                    if mxg > old_mxg:
+                        d = mxg - old_mxg
+                        result_slice_x = slice(d, -d)
+                        data_slice_x = slice(ix * mxsub, (ix + 1) * mxsub + 2 * old_mxg)
+                    else:
+                        d = old_mxg - mxg
+                        result_slice_x = slice(None)
+                        data_slice_x = slice(
+                            d + ix * mxsub, d + (ix + 1) * mxsub + 2 * mxg
+                        )
+
+                    if myg > old_myg:
+                        d = myg - old_myg
+                        result_slice_y = slice(d, -d)
+                        data_slice_y = slice(iy * mysub, (iy + 1) * mysub + 2 * old_myg)
+                    else:
+                        d = old_myg - myg
+                        result_slice_y = slice(None)
+                        data_slice_y = slice(
+                            d + iy * mysub, d + (iy + 1) * mysub + 2 * myg
+                        )
+
+                    result[result_slice_x, result_slice_y] = data[
+                        data_slice_x, data_slice_y
+                    ]
+
+                    return result
+                else:
+                    xoffset = old_mxg - mxg
+                    yoffset = old_myg - myg
+                    return data[
+                        xoffset + ix * mxsub : xoffset + (ix + 1) * mxsub + 2 * mxg,
+                        yoffset + iy * mysub : yoffset + (iy + 1) * mysub + 2 * myg,
+                    ]
+
             outfile = outfile_list[i]
             if v == "NPES":
                 outfile.write(v, npes)
@@ -622,28 +720,40 @@ def redistribute(npes, path="data", nxpe=None, output=".", informat=None, outfor
                 outfile.write(v, mysub)
             elif v == "MZSUB":
                 outfile.write(v, mzsub)
-            elif dimensions == ('x', 'y'):
+            elif v == "MXG":
+                outfile.write(v, mxg)
+            elif v == "MYG":
+                outfile.write(v, myg)
+            elif dimensions == ("x", "y"):
                 # Field2D
-                outfile.write(
-                    v, data[ix*mxsub:(ix+1)*mxsub+2*mxg, iy*mysub:(iy+1)*mysub+2*myg])
-            elif dimensions == ('x', 'z'):
+                outfile.write(v, get_block(data))
+            elif dimensions == ("x", "z"):
                 # FieldPerp
-                yindex_global = data.attributes['yindex_global']
-                if yindex_global + myg >= iy*mysub and yindex_global + myg < (iy+1)*mysub+2*myg:
-                    outfile.write(v, data[ix*mxsub:(ix+1)*mxsub+2*mxg, :])
+                yindex_global = data.attributes["yindex_global"]
+                if (
+                    yindex_global + myg >= iy * mysub
+                    and yindex_global + myg < (iy + 1) * mysub + 2 * myg
+                ):
+                    outfile.write(v, data[ix * mxsub : (ix + 1) * mxsub + 2 * mxg, :])
                 else:
-                    nullarray = BoutArray(np.zeros([mxsub+2*mxg, mysub+2*myg]), attributes={"bout_type":"FieldPerp", "yindex_global":-myg-1})
+                    nullarray = BoutArray(
+                        np.zeros([mxsub + 2 * mxg, mzsub]),
+                        attributes={
+                            "bout_type": "FieldPerp",
+                            "yindex_global": -myg - 1,
+                        },
+                    )
                     outfile.write(v, nullarray)
-            elif dimensions == ('x', 'y', 'z'):
+            elif dimensions == ("x", "y", "z"):
                 # Field3D
-                outfile.write(
-                    v, data[ix*mxsub:(ix+1)*mxsub+2*mxg, iy*mysub:(iy+1)*mysub+2*myg, :])
-            elif not any(d in dimensions for d in ('x', 'y', 'z')):
+                outfile.write(v, get_block(data))
+            elif not any(d in dimensions for d in ("x", "y", "z")):
                 # scalar or other non-spatially-dependent variable
                 outfile.write(v, data)
             else:
                 print(
-                    "ERROR: variable found with unexpected dimensions,", dimensions, v)
+                    "ERROR: variable found with unexpected dimensions,", dimensions, v
+                )
 
     f.close()
     for outfile in outfile_list:
@@ -687,7 +797,7 @@ def resizeY(newy, path="data", output=".", informat="nc", outformat=None, myg=2)
     if outformat is None:
         outformat = informat
 
-    file_list = glob.glob(os.path.join(path, "BOUT.restart.*."+informat))
+    file_list = glob.glob(os.path.join(path, "BOUT.restart.*." + informat))
 
     nfiles = len(file_list)
 
@@ -697,8 +807,8 @@ def resizeY(newy, path="data", output=".", informat="nc", outformat=None, myg=2)
 
     for i in range(nfiles):
         # Open each data file
-        infname = os.path.join(path, "BOUT.restart."+str(i)+"."+informat)
-        outfname = os.path.join(output, "BOUT.restart."+str(i)+"."+outformat)
+        infname = os.path.join(path, "BOUT.restart." + str(i) + "." + informat)
+        outfname = os.path.join(output, "BOUT.restart." + str(i) + "." + outformat)
 
         print("Processing %s -> %s" % (infname, outfname))
 
@@ -711,7 +821,7 @@ def resizeY(newy, path="data", output=".", informat="nc", outformat=None, myg=2)
             try:
                 # Convert to scalar if necessary
                 data = data[0]
-            except:
+            except (TypeError, IndexError):
                 pass
             outfile.write(var, data)
 
@@ -720,7 +830,7 @@ def resizeY(newy, path="data", output=".", informat="nc", outformat=None, myg=2)
 
         for var in varnames:
             dimensions = infile.dimensions(var)
-            if dimensions == ('x', 'y', 'z'):
+            if dimensions == ("x", "y", "z"):
                 # Could be an evolving variable [x,y,z]
 
                 print(" -> Resizing " + var)
@@ -731,19 +841,20 @@ def resizeY(newy, path="data", output=".", informat="nc", outformat=None, myg=2)
                 nx, ny, nz = indata.shape
 
                 # y coordinate in input and output data
-                iny = (arange(ny) - myg + 0.5) / (ny - 2*myg)
-                outy = (arange(newy) - myg + 0.5) / (newy - 2*myg)
+                iny = (arange(ny) - myg + 0.5) / (ny - 2 * myg)
+                outy = (arange(newy) - myg + 0.5) / (newy - 2 * myg)
 
                 outdata = zeros([nx, newy, nz])
 
                 for x in range(nx):
                     for z in range(nz):
                         f = interp1d(
-                            iny, indata[x, :, z], bounds_error=False, fill_value=0.0)
+                            iny, indata[x, :, z], bounds_error=False, fill_value=0.0
+                        )
                         outdata[x, :, z] = f(outy)
 
                 outfile.write(var, outdata)
-            elif dimensions == ('x', 'y'):
+            elif dimensions == ("x", "y"):
                 # Assume evolving variable [x,y]
                 print(" -> Resizing " + var)
 
@@ -753,14 +864,13 @@ def resizeY(newy, path="data", output=".", informat="nc", outformat=None, myg=2)
                 nx, ny = indata.shape
 
                 # y coordinate in input and output data
-                iny = (arange(ny) - myg + 0.5) / (ny - 2*myg)
-                outy = (arange(newy) - myg + 0.5) / (newy - 2*myg)
+                iny = (arange(ny) - myg + 0.5) / (ny - 2 * myg)
+                outy = (arange(newy) - myg + 0.5) / (newy - 2 * myg)
 
                 outdata = zeros([nx, newy])
 
                 for x in range(nx):
-                    f = interp1d(iny, indata[x, :],
-                                 bounds_error=False, fill_value=0.0)
+                    f = interp1d(iny, indata[x, :], bounds_error=False, fill_value=0.0)
                     outdata[x, :] = f(outy)
 
                 outfile.write(var, outdata)
@@ -773,7 +883,7 @@ def resizeY(newy, path="data", output=".", informat="nc", outformat=None, myg=2)
                 try:
                     # Convert to scalar if necessary
                     data = data[0]
-                except:
+                except (TypeError, IndexError):
                     pass
                 outfile.write(var, data)
 
